//
//  LLCoreDataManager.m
//  LLJogTest
//
//  Created by Lawrence Lomax on 17/01/2010.
//  Copyright 2010 Lawrence Lomax. All rights reserved.
//

#import "LLCoreDataManager.h"
#import "LLHelperMacros.h"

@implementation LLCoreDataManager

SYNTHESIZE_SINGLETON_FOR_CLASS(LLCoreDataManager)

#pragma mark Core Data stack
/**
 Returns the managed object context for the application.
 If the context doesn't already exist, it is created and bound to the persistent store coordinator for the application.
 */
- (NSManagedObjectContext *) managedObjectContext {
	
    if (managedObjectContext != nil) {
        return managedObjectContext;
    }
	
    NSPersistentStoreCoordinator *coordinator = [self persistentStoreCoordinator];
    if (coordinator != nil) {
        managedObjectContext = [[NSManagedObjectContext alloc] init];
        [managedObjectContext setPersistentStoreCoordinator: coordinator];
    }
    return managedObjectContext;
}


/**
 Returns the managed object model for the application.
 If the model doesn't already exist, it is created by merging all of the models found in the application bundle.
 */
- (NSManagedObjectModel *)managedObjectModel {
	
    if (managedObjectModel != nil) {
        return managedObjectModel;
    }
    managedObjectModel = [[NSManagedObjectModel mergedModelFromBundles:nil] retain];
    
    
    return managedObjectModel;
}


/**
 Returns the persistent store coordinator for the application.
 If the coordinator doesn't already exist, it is created and the application's store added to it.
 */
- (NSPersistentStoreCoordinator *)persistentStoreCoordinator {
	
    if (persistentStoreCoordinator != nil) {
        return persistentStoreCoordinator;
    }
	
    NSURL *storeUrl = [NSURL fileURLWithPath: [[self applicationDocumentsDirectory] stringByAppendingPathComponent: @"CoreDataDatabase.sqlite"]];
	//NSURL *storeUrl = [NSURL fileURLWithPath: [[self applicationDocumentsDirectory] stringByAppendingPathComponent: @"CoreDataBinaryDatabase.bin"]];
	
	
	NSError *error = nil;
    
    persistentStoreCoordinator = [[NSPersistentStoreCoordinator alloc] initWithManagedObjectModel:[self managedObjectModel]];
    if (![persistentStoreCoordinator addPersistentStoreWithType:NSSQLiteStoreType configuration:nil URL:storeUrl options:nil error:&error]) {
		/*
		 Replace this implementation with code to handle the error appropriately.
		 
		 abort() causes the application to generate a crash log and terminate. You should not use this function in a shipping application, although it may be useful during development. If it is not possible to recover from the error, display an alert panel that instructs the user to quit the application by pressing the Home button.
		 
		 Typical reasons for an error here include:
		 * The persistent store is not accessible
		 * The schema for the persistent store is incompatible with current managed object model
		 Check the error message to determine what the actual problem was.
		 */
		NSLog(@"Unresolved error %@, %@", error, [error userInfo]);
		abort();
    }    
	
    return persistentStoreCoordinator;
}

#pragma mark Application's Documents directory
/**
 Returns the path to the application's Documents directory.
 */
- (NSString *)applicationDocumentsDirectory {
	return [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) lastObject];
}

#pragma mark Context Management
- (NSManagedObject *)newObjectForName:(NSString *) entityName{
	if (!entityName) {
		return nil;
	}
	
	NSEntityDescription *entity = [[[self managedObjectModel] entitiesByName] objectForKey:entityName];
	
	if (!entity)
		return nil;
	
	NSManagedObject *object = [[NSManagedObject alloc] initWithEntity:entity insertIntoManagedObjectContext:[self managedObjectContext]];
	
	if (!object)
		return nil;
	
	return object;
}

-(NSError *)saveContextChanges{
	NSError * error = nil;
	[[self managedObjectContext] save:&error];
	if(error)
		[error autorelease];
	return error;
}

#pragma mark Fetch Requests
-(NSFetchRequest *)fetchRequestFromModelWithName:(NSString *)name{
	return [[self managedObjectModel] fetchRequestTemplateForName:name];
}

-(NSFetchRequest *)fetchRequestFromModelWithName:(NSString *)name withSubstitutions:(NSDictionary *)substitutions{
	return [[self managedObjectModel] fetchRequestFromTemplateWithName:name substitutionVariables:substitutions];
}

#pragma mark Fetching Objects with Requests
-(NSArray *)objectsWithFetchRequest:(NSFetchRequest *)fetchRequest{
	NSError *error = nil;
	NSArray *results = [[self managedObjectContext] executeFetchRequest:fetchRequest error:&error];
	
	//If there was an error then return nothing
	if (error) {
		NSLog(@"error:%@", error);
		return nil;
	}
	
	return results;
}

-(NSArray *)objectsWithEntityName:(NSString *)name withPredicate:(NSPredicate *)pred sortedWithDescriptors:(NSArray *)descriptors{
	NSFetchRequest * request = [[NSFetchRequest alloc] init];
	NSEntityDescription *entity = [[[self managedObjectModel] entitiesByName] objectForKey:name];
	[request setEntity:entity];
	[request setPredicate:pred];
	[request setSortDescriptors:descriptors];
	NSArray * results = [self objectsWithFetchRequest:request];
	[request release];
	return results;
}

-(NSArray *)objectsWithEntityName:(NSString *)name sortedWithDescriptors:(NSArray *)descriptors{
	return [self objectsWithEntityName:name withPredicate:nil sortedWithDescriptors:descriptors];
}

-(NSArray *)objectsWithEntityName:(NSString *)name{
	return [self objectsWithEntityName:name withPredicate:nil sortedWithDescriptors:nil];
}

-(NSManagedObject *)firstObjectWithFetchRequest:(NSFetchRequest *)fetchRequest{
	NSError * error = nil;
	NSArray * array = [[self managedObjectContext] executeFetchRequest:fetchRequest error:&error];
	if([array count] > 0)
		return (NSManagedObject *)[array objectAtIndex:0];
	else 
		return nil;
}

-(NSManagedObject *)firstObjectWithEntityName:(NSString *)name withPredicate:(NSPredicate *)pred sortedWithDescriptors:(NSArray *)descriptors{
	NSArray * results = [self objectsWithEntityName:name withPredicate:pred sortedWithDescriptors:descriptors];
	if(!results)
		return nil;
	if([results count] < 1)
		return nil;
	else
		return [results objectAtIndex:0];
}

-(NSManagedObject *)firstObjectWithEntityName:(NSString *)name sortedWithDescriptors:(NSArray *)descriptors{
	NSArray * results = [self objectsWithEntityName:name sortedWithDescriptors:descriptors];
	if(!results)
		return nil;
	if([results count] < 1)
		return nil;
	else
		return [results objectAtIndex:0];
}

-(NSManagedObject *)firstObjectWithEntityName:(NSString *)name{
	NSArray * results = [self objectsWithEntityName:name];
	if(!results)
		return nil;
	if([results count] < 1)
		return nil;
	else
		return [results objectAtIndex:0];
}


@end
